@@ -87,25 +87,14 @@
 /* Define to 1 if you have the <unistd.h> header file. */
 /* #undef HAVE_UNISTD_H */
 
-#include <xercesc/util/XercesVersion.hpp>
-
-#if (XERCES_VERSION_MAJOR >= 3)
-# define XMLTOOLING_XERCESC_COMPLIANT_DOMLS     1
-# define XMLTOOLING_XERCESC_BOOLSETIDATTRIBUTE  1
-# define XMLTOOLING_XERCESC_64BITSAFE           1
-# define XMLTOOLING_XERCESC_INPUTSTREAM_HAS_CONTENTTYPE 1
-#else
-# define XMLTOOLING_XERCESC_HAS_XMLBYTE_RELEASE 1
-#endif
-
-/* Define to 1 if you have the `xsecsize_t' type. */
-#define HAVE_XSECSIZE_T 1
-
 /* Define to 1 if you have the `curl_off_t' type. */
 #define HAVE_CURL_OFF_T 1
 
 /* Define to 1 if you have support for the CURLINFO_TLS_SSL_PTR feature. */
-#define HAVE_CURLINFO_TLS_SSL_PTR 1
+#define HAVE_DECL_CURLINFO_TLS_SSL_PTR 1
+
+/* Define to 1 if you have support for the CURLOPT_ACCEPT_ENCODING option. */
+#define HAVE_DECL_CURLOPT_ACCEPT_ENCODING 1
 
 /* Name of package */
 #define PACKAGE "xmltooling"
@@ -117,13 +106,13 @@
 #define PACKAGE_NAME "xmltooling"
 
 /* Define to the full name and version of this package. */
-#define PACKAGE_STRING "xmltooling 1.6.4"
+#define PACKAGE_STRING "xmltooling 3.0.0"
 
 /* Define to the one symbol short name of this package. */
 #define PACKAGE_TARNAME "xmltooling"
 
 /* Define to the version of this package. */
-#define PACKAGE_VERSION "1.6.4"
+#define PACKAGE_VERSION "3.0.0"
 
 /* Define to the necessary symbol if this constant uses a non-standard name on
    your system. */
@@ -136,27 +125,11 @@
 /* #undef TM_IN_SYS_TIME */
 
 /* Version number of package */
-#define VERSION "1.6.4"
+#define VERSION "3.0.0"
 
 /* Define if you wish to disable XML-Security-dependent features. */
 /* #undef XMLTOOLING_NO_XMLSEC */
 
-/* Define if you wish to disable Xalan-dependent features. */
-#define XSEC_NO_XALAN
-
-#ifndef XMLTOOLING_NO_XMLSEC
-# include <xsec/framework/XSECDefs.hpp>
-# if (_XSEC_VERSION_FULL >= 10500)
-#  define XMLTOOLING_XMLSEC_C14N11 1
-# endif
-# if (_XSEC_VERSION_FULL >= 10600)
-#  define XMLTOOLING_XMLSEC_MULTIPLECRL 1
-#  define XMLTOOLING_XMLSEC_SIGALGORITHM 1
-#  define XMLTOOLING_XMLSEC_ECC 1
-#  define XMLTOOLING_XMLSEC_DEBUGLOGGING 1
-# endif
-#endif
-
 /* Define to empty if `const' does not conform to ANSI C. */
 /* #undef const */
 
