@@ -59,6 +59,8 @@
 #ifndef XMLTOOLING_LITE
 # include <xsec/dsig/DSIGReference.hpp>
 # include <xsec/dsig/DSIGTransformList.hpp>
+# include <xsec/dsig/DSIGTransformEnvelope.hpp>
+# include <xsec/dsig/DSIGTransformC14n.hpp>
 using namespace xmlsignature;
 #endif
 
@@ -67,6 +69,8 @@ using namespace xmltooling;
 using namespace xercesc;
 using namespace std;
 
+using boost::scoped_ptr;
+
 static const XMLCh id[] =               UNICODE_LITERAL_2(i,d);
 static const XMLCh uri[] =              UNICODE_LITERAL_3(u,r,i);
 static const XMLCh url[] =              UNICODE_LITERAL_3(u,r,l);
@@ -79,43 +83,54 @@ static const XMLCh reloadChanges[] =    UNICODE_LITERAL_13(r,e,l,o,a,d,C,h,a,n,g
 static const XMLCh reloadInterval[] =   UNICODE_LITERAL_14(r,e,l,o,a,d,I,n,t,e,r,v,a,l);
 static const XMLCh maxRefreshDelay[] =  UNICODE_LITERAL_15(m,a,x,R,e,f,r,e,s,h,D,e,l,a,y);
 static const XMLCh backingFilePath[] =  UNICODE_LITERAL_15(b,a,c,k,i,n,g,F,i,l,e,P,a,t,h);
+
+#ifndef XMLTOOLING_LITE
 static const XMLCh type[] =             UNICODE_LITERAL_4(t,y,p,e);
 static const XMLCh certificate[] =      UNICODE_LITERAL_11(c,e,r,t,i,f,i,c,a,t,e);
 static const XMLCh signerName[] =       UNICODE_LITERAL_10(s,i,g,n,e,r,N,a,m,e);
 static const XMLCh _TrustEngine[] =     UNICODE_LITERAL_11(T,r,u,s,t,E,n,g,i,n,e);
 static const XMLCh _CredentialResolver[] = UNICODE_LITERAL_18(C,r,e,d,e,n,t,i,a,l,R,e,s,o,l,v,e,r);
-
-
-ReloadableXMLFile::ReloadableXMLFile(const DOMElement* e, Category& log, bool startReloadThread)
-    : m_root(e), m_local(true), m_validate(false), m_filestamp(0), m_reloadInterval(0),
-      m_lock(nullptr), m_log(log), m_loaded(false),
-#ifndef XMLTOOLING_LITE
-      m_credResolver(nullptr), m_trust(nullptr),
 #endif
-      m_shutdown(false), m_reload_wait(nullptr), m_reload_thread(nullptr)
+
+ReloadableXMLFile::ReloadableXMLFile(const DOMElement* e, Category& log, bool startReloadThread, bool deprecationSupport)
+    : m_root(e), m_local(true), m_validate(false), m_filestamp(0), m_reloadInterval(0),
+      m_log(log), m_loaded(false), m_shutdown(false)
 {
 #ifdef _DEBUG
     NDC ndc("ReloadableXMLFile");
 #endif
 
     // Establish source of data...
-    const XMLCh* source=e->getAttributeNS(nullptr,uri);
+    const XMLCh* source = e->getAttributeNS(nullptr, url);
     if (!source || !*source) {
-        source=e->getAttributeNS(nullptr,url);
+        if (deprecationSupport)
+            source = e->getAttributeNS(nullptr, uri);
         if (!source || !*source) {
-            source=e->getAttributeNS(nullptr,path);
+            source = e->getAttributeNS(nullptr, path);
             if (!source || !*source) {
-                source=e->getAttributeNS(nullptr,pathname);
-                if (!source || !*source) {
-                    source=e->getAttributeNS(nullptr,file);
+                if (deprecationSupport) {
+                    source = e->getAttributeNS(nullptr, pathname);
                     if (!source || !*source) {
-                        source=e->getAttributeNS(nullptr,filename);
+                        source = e->getAttributeNS(nullptr, file);
+                        if (!source || !*source) {
+                            source = e->getAttributeNS(nullptr, filename);
+                            if (source && *source) {
+                                m_log.warn("DEPRECATED: filename attribute should be replaced with path to specify local resource");
+                            }
+                        }
+                        else {
+                            m_log.warn("DEPRECATED: file attribute should be replaced with path to specify local resource");
+                        }
+                    }
+                    else {
+                        m_log.warn("DEPRECATED: pathname attribute should be replaced with path to specify local resource");
                     }
                 }
             }
         }
         else {
-            m_local=false;
+            m_local = false;
+            m_log.warn("DEPRECATED: uri attribute should be replaced with url to specify remote resource");
         }
     }
     else {
@@ -128,8 +143,8 @@ ReloadableXMLFile::ReloadableXMLFile(const DOMElement* e, Category& log, bool st
         auto_ptr_char temp(source);
         m_source = temp.get();
 
-        if (!m_local && !strstr(m_source.c_str(),"://")) {
-            log.warn("deprecated usage of uri/url attribute for a local resource, use path instead");
+        if (deprecationSupport && !m_local && !strstr(m_source.c_str(),"://")) {
+            log.warn("DEPRECATED: usage of uri/url attribute for a local resource, use path instead");
             m_local = true;
         }
 
@@ -137,23 +152,23 @@ ReloadableXMLFile::ReloadableXMLFile(const DOMElement* e, Category& log, bool st
         // Check for signature bits.
         if (e->hasAttributeNS(nullptr, certificate)) {
             // Use a file-based credential resolver rooted here.
-            m_credResolver = XMLToolingConfig::getConfig().CredentialResolverManager.newPlugin(FILESYSTEM_CREDENTIAL_RESOLVER, e);
+            m_credResolver.reset(XMLToolingConfig::getConfig().CredentialResolverManager.newPlugin(FILESYSTEM_CREDENTIAL_RESOLVER, e, deprecationSupport));
         }
         else {
             const DOMElement* sub = XMLHelper::getFirstChildElement(e, _CredentialResolver);
             string t(XMLHelper::getAttrString(sub, nullptr, type));
             if (!t.empty()) {
-                m_credResolver = XMLToolingConfig::getConfig().CredentialResolverManager.newPlugin(t.c_str(), sub);
+                m_credResolver.reset(XMLToolingConfig::getConfig().CredentialResolverManager.newPlugin(t.c_str(), sub, deprecationSupport));
             }
             else {
                 sub = XMLHelper::getFirstChildElement(e, _TrustEngine);
                 t = XMLHelper::getAttrString(sub, nullptr, type);
                 if (!t.empty()) {
-                    TrustEngine* trust = XMLToolingConfig::getConfig().TrustEngineManager.newPlugin(t.c_str(), sub);
-                    if (!(m_trust = dynamic_cast<SignatureTrustEngine*>(trust))) {
-                        delete trust;
+                    auto_ptr<TrustEngine> trust(XMLToolingConfig::getConfig().TrustEngineManager.newPlugin(t.c_str(), sub, deprecationSupport));
+                    if (!dynamic_cast<SignatureTrustEngine*>(trust.get())) {
                         throw XMLToolingException("TrustEngine-based ReloadableXMLFile requires a SignatureTrustEngine plugin.");
                     }
+                    m_trust.reset(dynamic_cast<SignatureTrustEngine*>(trust.release()));
 
                     m_signerName = XMLHelper::getAttrString(e, nullptr, signerName);
                 }
@@ -176,7 +191,7 @@ ReloadableXMLFile::ReloadableXMLFile(const DOMElement* e, Category& log, bool st
                     m_filestamp = stat_buf.st_mtime;
                 else
                     throw IOException("Unable to access local file ($1)", params(1,m_source.c_str()));
-                m_lock = RWLock::create();
+                m_lock.reset(RWLock::create());
             }
             FILE* cfile = fopen(m_source.c_str(), "r");
             if (cfile)
@@ -202,15 +217,15 @@ ReloadableXMLFile::ReloadableXMLFile(const DOMElement* e, Category& log, bool st
                         }
                     }
                 }
-                catch (exception&) {
+                catch (const exception&) {
                 }
             }
-            m_reloadInterval = XMLHelper::getAttrInt(e, 0, reloadInterval);
+            m_reloadInterval = XMLHelper::getAttrInt(e, 0, maxRefreshDelay);
             if (m_reloadInterval == 0)
-                m_reloadInterval = XMLHelper::getAttrInt(e, 0, maxRefreshDelay);
+                m_reloadInterval = XMLHelper::getAttrInt(e, 0, reloadInterval);
             if (m_reloadInterval > 0) {
                 m_log.debug("will reload remote resource at most every %d seconds", m_reloadInterval);
-                m_lock = RWLock::create();
+                m_lock.reset(RWLock::create());
             }
             m_filestamp = time(nullptr);   // assume it gets loaded initially
         }
@@ -218,8 +233,13 @@ ReloadableXMLFile::ReloadableXMLFile(const DOMElement* e, Category& log, bool st
         if (startReloadThread)
             startup();
     }
+    else if (e && XMLHelper::getFirstChildElement(e)) {
+        // TODO: what we want to do here is delegate a check to the subclasses to immediately check for the "right"
+        // child element so we can catch the deprecated syntax mistake here.
+        log.info("no resource url/path supplied, assuming inline configuration");
+    }
     else {
-        log.debug("no resource uri/path/name supplied, will load inline configuration");
+        throw XMLToolingException("XML configuration resource missing url/path attributes and has no inline content");
     }
 
     m_id = XMLHelper::getAttrString(e, nullptr, id);
@@ -228,14 +248,13 @@ ReloadableXMLFile::ReloadableXMLFile(const DOMElement* e, Category& log, bool st
 ReloadableXMLFile::~ReloadableXMLFile()
 {
     shutdown();
-    delete m_lock;
 }
 
 void ReloadableXMLFile::startup()
 {
     if (m_lock && !m_reload_thread) {
-        m_reload_wait = CondWait::create();
-        m_reload_thread = Thread::create(&reload_fn, this);
+        m_reload_wait.reset(CondWait::create());
+        m_reload_thread.reset(Thread::create(&reload_fn, this));
     }
 }
 
@@ -246,10 +265,8 @@ void ReloadableXMLFile::shutdown()
         m_shutdown = true;
         m_reload_wait->signal();
         m_reload_thread->join(nullptr);
-        delete m_reload_thread;
-        delete m_reload_wait;
-        m_reload_thread = nullptr;
-        m_reload_wait = nullptr;
+        m_reload_thread.reset();
+        m_reload_wait.reset();
     }
 }
 
@@ -272,13 +289,13 @@ void* ReloadableXMLFile::reload_fn(void* pv)
     NDC ndc("reload");
 #endif
 
-    auto_ptr<Mutex> mutex(Mutex::create());
+    scoped_ptr<Mutex> mutex(Mutex::create());
     mutex->lock();
 
     if (r->m_local)
-        r->m_log.info("reload thread started...running when signaled");
+        r->m_log.debug("reload thread started...running when signaled");
     else
-        r->m_log.info("reload thread started...running every %d seconds", r->m_reloadInterval);
+        r->m_log.debug("reload thread started...running every %d seconds", r->m_reloadInterval);
 
     while (!r->m_shutdown) {
         if (r->m_local)
@@ -287,25 +304,27 @@ void* ReloadableXMLFile::reload_fn(void* pv)
             r->m_reload_wait->timedwait(mutex.get(), r->m_reloadInterval);
         if (r->m_shutdown)
             break;
-		
-#ifdef WIN32
-        struct _stat stat_buf;
-        if (_stat(r->m_source.c_str(), &stat_buf) != 0)
-            continue;
-#else
-        struct stat stat_buf;
-        if (stat(r->m_source.c_str(), &stat_buf) != 0)
-            continue;
-#endif
-        if (r->m_filestamp >= stat_buf.st_mtime)
-            continue;
 
-        // Elevate lock and recheck.
-        r->m_log.debug("timestamp of local resource changed, obtaining write lock");
-        r->m_lock->wrlock();
-        r->m_filestamp = stat_buf.st_mtime;
-        r->m_log.debug("timestamp of local resource changed, releasing write lock");
-        r->m_lock->unlock();
+        if (r->m_local) {
+#ifdef WIN32
+            struct _stat stat_buf;
+            if (_stat(r->m_source.c_str(), &stat_buf) != 0)
+                continue;
+#else
+            struct stat stat_buf;
+            if (stat(r->m_source.c_str(), &stat_buf) != 0)
+                continue;
+#endif
+            if (r->m_filestamp >= stat_buf.st_mtime)
+                continue;
+
+            // Grab lock to protect m_filestamp.
+            r->m_log.debug("timestamp of local resource changed, obtaining write lock");
+            r->m_lock->wrlock();
+            r->m_filestamp = stat_buf.st_mtime;
+            r->m_log.debug("timestamp of local resource changed, releasing write lock");
+            r->m_lock->unlock();
+        }
 		
         try {
             r->m_log.info("reloading %s resource...", r->m_local ? "local" : "remote");
@@ -313,7 +332,7 @@ void* ReloadableXMLFile::reload_fn(void* pv)
             if (ret.first)
                 ret.second->getOwnerDocument()->release();
         }
-        catch (long& ex) {
+        catch (const long& ex) {
             if (ex == HTTPResponse::XMLTOOLING_HTTP_STATUS_NOTMODIFIED) {
                 r->m_log.info("remote resource (%s) unchanged from cached version", r->m_source.c_str());
             }
@@ -322,12 +341,12 @@ void* ReloadableXMLFile::reload_fn(void* pv)
                 r->m_log.crit("maintaining existing configuration, remote resource fetch returned atypical status code (%d)", ex);
             }
         }
-        catch (exception& ex) {
+        catch (const exception& ex) {
             r->m_log.crit("maintaining existing configuration, error reloading resource (%s): %s", r->m_source.c_str(), ex.what());
         }
     }
 
-    r->m_log.info("reload thread finished");
+    r->m_log.debug("reload thread finished");
 
     mutex->unlock();
 
@@ -377,11 +396,6 @@ void ReloadableXMLFile::unlock()
         m_lock->unlock();
 }
 
-pair<bool, DOMElement*> ReloadableXMLFile::load(bool backup)
-{
-    return load(backup, "");
-}
-
 pair<bool,DOMElement*> ReloadableXMLFile::load(bool backup, string backingFile)
 {
 #ifdef _DEBUG
@@ -450,7 +464,7 @@ pair<bool,DOMElement*> ReloadableXMLFile::load(bool backup, string backingFile)
                         throw XMLSecurityException("Signature validation required, but no signature found.");
 
                     // Wrap and unmarshall the signature for the duration of the check.
-                    auto_ptr<Signature> sigobj(dynamic_cast<Signature*>(SignatureBuilder::buildOneFromElement(sigel)));    // don't bind to document
+                    scoped_ptr<Signature> sigobj(dynamic_cast<Signature*>(SignatureBuilder::buildOneFromElement(sigel)));    // don't bind to document
                     validateSignature(*sigobj.get());
                 }
                 catch (exception&) {
@@ -481,7 +495,7 @@ pair<bool,DOMElement*> ReloadableXMLFile::load(bool backup, string backingFile)
             remove(backingFile.c_str());
         throw;
     }
-    catch (long& ex) {
+    catch (long&) {
         if (!backingFile.empty() && !backup)
             remove(backingFile.c_str());
         throw;
@@ -494,7 +508,7 @@ pair<bool,DOMElement*> ReloadableXMLFile::load()
     // backup of a remote resource (if available), and for backing up remote
     // resources.
     try {
-        pair<bool,DOMElement*> ret = load(false);
+        pair<bool,DOMElement*> ret = load(false, m_backing);
         if (!m_backing.empty()) {
             m_log.debug("backing up remote resource to (%s)", m_backing.c_str());
             try {
@@ -503,13 +517,13 @@ pair<bool,DOMElement*> ReloadableXMLFile::load()
                 backer << *(ret.second->getOwnerDocument());
                 preserveCacheTag();
             }
-            catch (exception& ex) {
+            catch (const exception& ex) {
                 m_log.crit("exception while backing up resource: %s", ex.what());
             }
         }
         return ret;
     }
-    catch (long& responseCode) {
+    catch (long responseCode) {
         // If there's an HTTP error or the document hasn't changed,
         // use the backup iff we have no "valid" resource in place.
         // That prevents reload of the backup copy any time the document
@@ -517,13 +531,13 @@ pair<bool,DOMElement*> ReloadableXMLFile::load()
         if (responseCode == HTTPResponse::XMLTOOLING_HTTP_STATUS_NOTMODIFIED)
             m_log.info("remote resource (%s) unchanged from cached version", m_source.c_str());
         if (!m_loaded && !m_backing.empty())
-            return load(true);
+            return load(true, "");
         throw;
     }
-    catch (exception&) {
+    catch (const exception&) {
         // Same as above, but for general load/parse errors.
         if (!m_loaded && !m_backing.empty())
-            return load(true);
+            return load(true, "");
         throw;
     }
 }
@@ -560,30 +574,27 @@ void ReloadableXMLFile::preserveCacheTag()
 
 void ReloadableXMLFile::validateSignature(Signature& sigObj) const
 {
-    DSIGSignature* sig=sigObj.getXMLSignature();
+    const DSIGSignature* sig=sigObj.getXMLSignature();
     if (!sig)
         throw XMLSecurityException("Signature does not exist yet.");
 
     // Make sure the whole document was signed.
     bool valid=false;
-    DSIGReferenceList* refs=sig->getReferenceList();
+    const DSIGReferenceList* refs=sig->getReferenceList();
     if (refs && refs->getSize()==1) {
-        DSIGReference* ref=refs->item(0);
+        const DSIGReference* ref=refs->item(0);
         if (ref) {
             const XMLCh* URI=ref->getURI();
             if (URI==nullptr || *URI==0) {
-                DSIGTransformList* tlist=ref->getTransforms();
+                const DSIGTransformList* tlist=ref->getTransforms();
                 if (tlist->getSize() <= 2) { 
                     for (unsigned int i=0; tlist && i<tlist->getSize(); i++) {
-                        if (tlist->item(i)->getTransformType()==TRANSFORM_ENVELOPED_SIGNATURE)
+                        const DSIGTransform* t = tlist->item(i);
+                        if (dynamic_cast<const DSIGTransformEnvelope*>(t)) {
                             valid=true;
-                        else if (tlist->item(i)->getTransformType()!=TRANSFORM_EXC_C14N &&
-                                 tlist->item(i)->getTransformType()!=TRANSFORM_C14N
-#ifdef XMLTOOLING_XMLSEC_C14N11
-                                 && tlist->item(i)->getTransformType()!=TRANSFORM_C14N11
-#endif
-                                 ) {
-                            valid=false;
+                        }
+                        else if (!dynamic_cast<const DSIGTransformC14n*>(t)) {
+                            valid = false;
                             break;
                         }
                     }
@@ -603,7 +614,7 @@ void ReloadableXMLFile::validateSignature(Signature& sigObj) const
         cc.setPeerName(m_signerName.c_str());
 
     if (m_credResolver) {
-        Locker locker(m_credResolver);
+        Locker locker(m_credResolver.get());
         vector<const Credential*> creds;
         if (m_credResolver->resolve(creds, &cc)) {
             SignatureValidator sigValidator;
@@ -613,7 +624,7 @@ void ReloadableXMLFile::validateSignature(Signature& sigObj) const
                     sigValidator.validate(&sigObj);
                     return; // success!
                 }
-                catch (exception&) {
+                catch (const exception&) {
                 }
             }
             throw XMLSecurityException("Unable to verify signature with supplied key(s).");
@@ -623,10 +634,10 @@ void ReloadableXMLFile::validateSignature(Signature& sigObj) const
         }
     }
     else if (m_trust) {
-        auto_ptr<CredentialResolver> dummy(
-            XMLToolingConfig::getConfig().CredentialResolverManager.newPlugin(DUMMY_CREDENTIAL_RESOLVER, nullptr)
+        scoped_ptr<CredentialResolver> dummy(
+            XMLToolingConfig::getConfig().CredentialResolverManager.newPlugin(DUMMY_CREDENTIAL_RESOLVER, nullptr, false)
             );
-        if (m_trust->validate(sigObj, *(dummy.get()), &cc))
+        if (m_trust->validate(sigObj, *dummy, &cc))
             return;
         throw XMLSecurityException("TrustEngine unable to verify signature.");
     }
